function grid = metropolis(N,kT,J,t,grid)
% Metropolis sampling for the 2D Ising model

% plot initial configuration
M = sum(sum(grid))/numel(grid);
E = isingenergy(grid,J);
figure(1);
isingplot(grid,N,J,kT,M,E);

% Pick a sequence of random spins (with a linear index)
spin = randi(N,2,t);
% Precompute the possible values of exp(-dE)
p=exp(-2 * J / kT*[-4:2:4]);
% Evolve the system for a fixed number of steps
k=0;
for s_ij=spin
    % location of ith spin
    i = s_ij(1);
    j = s_ij(2);
    % Calculate the change in energy of flipping s
    dE =  grid(i, j) * (...
        grid(mod(i    , N) + 1, j) + ...
        grid(mod(i - 2, N) + 1, j) + ...
        grid(i, mod(j    , N) + 1) + ...
        grid(i, mod(j - 2, N) + 1) ...
    );
    % Decide if a transition will occur
    if rand <= p(dE/2+3), grid(i, j) = -1*grid(i, j); end
    % Refresh display of current spin configuration every N^2 trials
    k=k+1;
    if mod(k,N^2)==0
       % Sum up our variables of interest and plot
        M = sum(sum(grid))/numel(grid);
        E = isingenergy(grid,J);
        isingplot(grid,N,J,kT,M,E);
    end
end

end